package tokenize

import (
	"testing"
)

func TestMoveCursorByRunes(t *testing.T) {
	tokenAPI := NewAPI("")
	tokenAPI.Rune.MoveCursor('ɹ')
	tokenAPI.Rune.MoveCursor('n')
	tokenAPI.Rune.MoveCursor('u')
	tokenAPI.Rune.MoveCursor('\r')
	tokenAPI.Rune.MoveCursor('\n')
	tokenAPI.Rune.MoveCursor('ǝ')

	AssertEqual(t, "line 2, column 2", tokenAPI.Input.Cursor(), "Cursor position after moving by rune")
	AssertEqual(t, 8, tokenAPI.pointers.offset, "Offset after moving by rune")
}

func TestWhenMovingCursor_CursorPositionIsUpdated(t *testing.T) {
	for _, test := range []struct {
		name   string
		input  []string
		byte   int
		rune   int
		line   int
		column int
	}{
		{"No input at all", []string{""}, 0, 0, 0, 0},
		{"One ASCII char", []string{"a"}, 1, 1, 0, 1},
		{"Multiple ASCII chars", []string{"abc"}, 3, 3, 0, 3},
		{"One newline", []string{"\n"}, 1, 1, 1, 0},
		{"Carriage return", []string{"\r\r\r"}, 3, 3, 0, 3},
		{"One UTF8 3 byte char", []string{"⌘"}, 3, 1, 0, 1},
		{"Mixture", []string{"Hello\n\npretty\nW⌘O⌘R⌘L⌘D"}, 31, 23, 3, 9},
		{"Multiple calls", []string{"hello", "world"}, 10, 10, 0, 10},
	} {
		tokenAPI := NewAPI("")
		for _, s := range test.input {
			for _, r := range s {
				tokenAPI.Rune.MoveCursor(r)
			}
		}
		if tokenAPI.pointers.line != test.line {
			t.Errorf("[%s] Unexpected line offset %d (expected %d)", test.name, tokenAPI.pointers.line, test.line)
		}
		if tokenAPI.pointers.column != test.column {
			t.Errorf("[%s] Unexpected column offset %d (expected %d)", test.name, tokenAPI.pointers.column, test.column)
		}
	}
}
